class Release < ActiveRecord::Base
  include Redmine::SafeAttributes
  
  belongs_to :project
  belongs_to :version
  has_many :test_executions
  
  acts_as_customizable
  acts_as_attachable :view_permission => :view_files,
                     :delete_permission => :manage_files

  RELEASE_STATUSES = %w( new in_progress failed pass cancelled)

  validates_presence_of :version
  validates_presence_of :name
  validates_uniqueness_of :name, :scope => [:project_id]
  validates_length_of :name, :maximum => 60
  validates_format_of :target_date, :with => /\d{4}-\d{2}-\d{2}/, :message => :not_a_date, :allow_nil => true
  validates_inclusion_of :status, :in => RELEASE_STATUSES

  has_and_belongs_to_many :issues

  scope :open, lambda { where(:status => "open") }

#  def issues
#    version ? version.fixed_issues : []
#  end

  def latest_execution
    test_executions.empty? ? nil : test_executions[-1]
  end

  
  # Returns true if +user+ or current user is allowed to view the version
  def visible?(user=User.current)
    user.allowed_to?(:view_issues, self.project)
  end

  def due_date
    target_date
  end

  # release's issues are set at release's creation time
  def refresh_issues(save = true)
    the_project = self.project
    # current_issues = self.issues
    # 
    # closed_issues = the_project.issues.select { |x| x.closed? || x.status.name != "Rejected" }
    # self.issues << (closed_issues - current_issues)
    # self.issues << (self.version.fixed_issues - self.issues ) if self.version
    closed_issues = the_project.issues.select { |x| x.closed?}
		self.issues = (closed_issues + self.version.fixed_issues).select{|x| x.status.name != "Rejected"}.uniq
    self.save if save
  end


  def done?
    status == 'failed' || status == 'pass' || status == 'cancelled'
  end

  def open?
    status == 'open'
  end

  # Returns true if the version is completed: due date reached and no open issues
  def completed?
    target_date && (target_date <= Date.today) && (open_issues_count == 0)
  end

  # Returns true if the version is overdue: due date reached and some open issues
  def overdue?
    target_date && (target_date < Date.today) && (open_issues_count > 0)
  end

  def to_s;
    name
  end

  # Versions are sorted by target_date and "Project Name - Version name"
  # Those with no target_date are at the end, sorted by "Project Name - Version name"
  def <=>(release)
    if self.target_date
      if release.target_date
        if self.target_date == release.target_date
          "#{self.project.name} - #{self.name}" <=> "#{release.project.name} - #{release.name}"
        else
          self.target_date <=> release.target_date
        end
      else
        -1
      end
    else
      if release.target_date
        1
      else
        "#{self.project.name} - #{self.name}" <=> "#{release.project.name} - #{release.name}"
      end
    end
  end


  private
end
